/*
    GNOME Commander - A GNOME based file manager 
    Copyright (C) 2001-2002 Marcus Bjurman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/ 
#include <config.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include "gnome-cmd-includes.h"

typedef struct
{
	GnomeCmdFile *finfo;
	gchar *new_name;
} RenameEntry;


typedef struct
{
	gchar *from;
	gchar *to;
} PatternEntry;


struct _GnomeCmdAdvrenameDialogPrivate
{
	GList *files;
	GList *entries;
	GList *patterns;
};


static GnomeDialogClass *parent_class = NULL;


static void
free_data (GnomeCmdAdvrenameDialog *dialog)
{
	GList *tmp;
	
	gnome_cmd_file_list_free (dialog->priv->files);

	tmp = dialog->priv->entries;
	while (tmp) {
		RenameEntry *entry = (RenameEntry*)tmp->data;
		g_free (entry->new_name);
		g_free (entry);
		tmp = tmp->next;
	}
	g_list_free (dialog->priv->entries);

	tmp = dialog->priv->patterns;
	while (tmp) {
		PatternEntry *entry = (PatternEntry*)tmp->data;
		g_free (entry->from);
		g_free (entry->to);
		g_free (entry);
		tmp = tmp->next;
	}
	g_list_free (dialog->priv->patterns);

	g_free (dialog->priv);
	dialog->priv = NULL;
}


static RenameEntry *
rename_entry_new (void)
{
	RenameEntry *entry = g_new (RenameEntry, 1);
	entry->finfo = NULL;
	entry->new_name = NULL;
	return entry;
}


static PatternEntry *
get_pattern_entry_from_row (GnomeCmdAdvrenameDialog *dialog, gint row)
{
	GtkWidget *pat_list = lookup_widget (GTK_WIDGET (dialog), "pat_list");
	return gtk_clist_get_row_data (GTK_CLIST (pat_list), row);
}


static gint 
get_row_from_rename_entry (GnomeCmdAdvrenameDialog *dialog, RenameEntry *entry)
{
	GtkWidget *res_list = lookup_widget (GTK_WIDGET (dialog), "res_list");
	return gtk_clist_find_row_from_data (GTK_CLIST (res_list), entry);
}


static void
add_rename_entry (GnomeCmdAdvrenameDialog *dialog, GnomeCmdFile *finfo)
{
	gint row;
	gchar *text[3];
	GtkWidget *res_list = lookup_widget (GTK_WIDGET (dialog), "res_list");
	RenameEntry *entry = rename_entry_new ();

	entry->finfo = finfo;

	text[0] = finfo->info->name;
	text[1] = NULL;
	text[2] = NULL;

	row = gtk_clist_append (GTK_CLIST (res_list), text);
	gtk_clist_set_row_data (GTK_CLIST (res_list), row, entry);
	dialog->priv->entries = g_list_append (dialog->priv->entries, entry);
}


static void
add_pattern_entry (GnomeCmdAdvrenameDialog *dialog, gchar *from, gchar *to)
{
	gint row;
	gchar *text[3];
	GtkWidget *pat_list = lookup_widget (GTK_WIDGET (dialog), "pat_list");
	PatternEntry *entry = g_new (PatternEntry, 1);

	entry->to = to;
	entry->from = from;

	text[0] = from;
	text[1] = to;
	text[2] = NULL;

	row = gtk_clist_append (GTK_CLIST (pat_list), text);
	gtk_clist_set_row_data (GTK_CLIST (pat_list), row, entry);
	dialog->priv->patterns = g_list_append (dialog->priv->patterns, entry);
}


static void
on_rule_dialog_ok (GtkButton *button, GnomeCmdAdvrenameDialog *dialog)
{
	GtkWidget *rule_dialog, *find_entry, *replace_entry;

	rule_dialog = lookup_widget (GTK_WIDGET (button), "rule_dialog");
	find_entry = lookup_widget (GTK_WIDGET (button), "find_entry");
	replace_entry = lookup_widget (GTK_WIDGET (button), "replace_entry");
	
	add_pattern_entry (
		dialog,
		gtk_entry_get_text (GTK_ENTRY (find_entry)),
		gtk_entry_get_text (GTK_ENTRY (replace_entry)));

	gtk_widget_hide (rule_dialog);
}


static void
on_rule_dialog_cancel (GtkButton *button, GnomeCmdAdvrenameDialog *dialog)
{
	GtkWidget *rule_dialog = lookup_widget (GTK_WIDGET (button), "rule_dialog");
	gtk_widget_hide (rule_dialog);
}


static GtkWidget*
create_rule_dialog (GnomeCmdAdvrenameDialog *parent_dialog)
{
	GtkWidget *rule_dialog;
	GtkWidget *dialog_vbox2;
	GtkWidget *table1;
	GtkWidget *label7;
	GtkWidget *label8;
	GtkWidget *find_entry;
	GtkWidget *replace_entry;
	GtkWidget *dialog_action_area2;
	GtkWidget *ok_button;
	GtkWidget *cancel_button;

	rule_dialog = gnome_dialog_new (_("New rule"), NULL);
	gtk_object_set_data (GTK_OBJECT (rule_dialog), "rule_dialog", rule_dialog);
	gtk_window_set_policy (GTK_WINDOW (rule_dialog), FALSE, FALSE, FALSE);

	dialog_vbox2 = GNOME_DIALOG (rule_dialog)->vbox;
	gtk_object_set_data (GTK_OBJECT (rule_dialog), "dialog_vbox2", dialog_vbox2);
	gtk_widget_show (dialog_vbox2);

	table1 = gtk_table_new (2, 2, FALSE);
	gtk_widget_ref (table1);
	gtk_object_set_data_full (GTK_OBJECT (rule_dialog), "table1", table1,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (table1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox2), table1, TRUE, TRUE, 0);
	gtk_table_set_row_spacings (GTK_TABLE (table1), 2);
	gtk_table_set_col_spacings (GTK_TABLE (table1), 2);

	label7 = gtk_label_new (_("Replace this"));
	gtk_widget_ref (label7);
	gtk_object_set_data_full (GTK_OBJECT (rule_dialog), "label7", label7,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label7);
	gtk_table_attach (GTK_TABLE (table1), label7, 0, 1, 0, 1,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label7), 0, 0.5);

	label8 = gtk_label_new (_("With this"));
	gtk_widget_ref (label8);
	gtk_object_set_data_full (GTK_OBJECT (rule_dialog), "label8", label8,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label8);
	gtk_table_attach (GTK_TABLE (table1), label8, 0, 1, 1, 2,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label8), 0, 0.5);

	find_entry = gtk_entry_new ();
	gtk_widget_ref (find_entry);
	gtk_object_set_data_full (GTK_OBJECT (rule_dialog), "find_entry", find_entry,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (find_entry);
	gtk_table_attach (GTK_TABLE (table1), find_entry, 1, 2, 0, 1,
					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);

	replace_entry = gtk_entry_new ();
	gtk_widget_ref (replace_entry);
	gtk_object_set_data_full (GTK_OBJECT (rule_dialog), "replace_entry", replace_entry,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (replace_entry);
	gtk_table_attach (GTK_TABLE (table1), replace_entry, 1, 2, 1, 2,
					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					  (GtkAttachOptions) (0), 0, 0);

	dialog_action_area2 = GNOME_DIALOG (rule_dialog)->action_area;
	gtk_object_set_data (GTK_OBJECT (rule_dialog), "dialog_action_area2", dialog_action_area2);
	gtk_widget_show (dialog_action_area2);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area2), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area2), 8);

	gnome_dialog_append_button (GNOME_DIALOG (rule_dialog), GNOME_STOCK_BUTTON_OK);
	ok_button = GTK_WIDGET (g_list_last (GNOME_DIALOG (rule_dialog)->buttons)->data);
	gtk_widget_ref (ok_button);
	gtk_object_set_data_full (GTK_OBJECT (rule_dialog), "ok_button", ok_button,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (ok_button);
	GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
	
	gnome_dialog_append_button (GNOME_DIALOG (rule_dialog), GNOME_STOCK_BUTTON_CANCEL);
	cancel_button = GTK_WIDGET (g_list_last (GNOME_DIALOG (rule_dialog)->buttons)->data);
	gtk_widget_ref (cancel_button);
	gtk_object_set_data_full (GTK_OBJECT (rule_dialog), "cancel_button", cancel_button,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (cancel_button);
	GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);

	gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
						GTK_SIGNAL_FUNC (on_rule_dialog_ok),
						parent_dialog);
	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
						GTK_SIGNAL_FUNC (on_rule_dialog_cancel),
						parent_dialog);
	
	return rule_dialog;
}


static gchar *
create_new_name (const gchar *name, GList *patterns)
{
	gchar *new_name = g_strdup (name);
	
	while (patterns)
	{
		PatternEntry *entry = (PatternEntry*)patterns->data;
		gchar **ret;
		gchar *tmp = new_name;

		ret = g_strsplit (new_name, entry->from, 0);
		new_name = g_strjoinv (entry->to, ret); 

		g_free (tmp);
		g_strfreev (ret);
		
		patterns = patterns->next;
	}

	return new_name;
}

static void update_new_names (GnomeCmdAdvrenameDialog *dialog)
{
	GList *tmp = dialog->priv->entries;

	while (tmp) {
		RenameEntry *entry = (RenameEntry*)tmp->data;
		
		GList *patterns = dialog->priv->patterns;

		entry->new_name = create_new_name (entry->finfo->info->name, patterns);
		
		tmp = tmp->next;
	}
}


static void redisplay_new_names (GnomeCmdAdvrenameDialog *dialog)
{
	GList *tmp = dialog->priv->entries;
	GtkWidget *res_list = lookup_widget (GTK_WIDGET (dialog), "res_list");

	while (tmp) {
		RenameEntry *entry = (RenameEntry*)tmp->data;

		gtk_clist_set_text (GTK_CLIST (res_list),
							get_row_from_rename_entry (dialog, entry),
							1,
							entry->new_name);
		
		tmp = tmp->next;
	}
}


static void
change_names (GnomeCmdAdvrenameDialog *dialog)
{
	GList *tmp = dialog->priv->entries;

	while (tmp) {
		RenameEntry *entry = (RenameEntry*)tmp->data;

		if (strcmp (entry->finfo->info->name, entry->new_name) != 0)
			gnome_cmd_file_rename (entry->finfo, entry->new_name);
		tmp = tmp->next;
	}
}


static void on_add (GtkButton *button, GnomeCmdAdvrenameDialog *dialog)
{
	GtkWidget *rule_dialog;

	rule_dialog = create_rule_dialog (dialog);
	gtk_widget_ref (rule_dialog);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "rule_dialog", rule_dialog,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rule_dialog);
}


static void on_remove (GtkButton *button, GnomeCmdAdvrenameDialog *dialog)
{
	GtkWidget *pat_list = lookup_widget (GTK_WIDGET (dialog), "pat_list");
	PatternEntry *entry = get_pattern_entry_from_row (dialog, GTK_CLIST (pat_list)->focus_row);

	if (entry) {
		dialog->priv->patterns = g_list_remove (dialog->priv->patterns, entry);
		gtk_clist_remove (GTK_CLIST (pat_list), GTK_CLIST (pat_list)->focus_row);
	}
}


static void on_test (GtkButton *button, GnomeCmdAdvrenameDialog *dialog)
{
	update_new_names (dialog);
	redisplay_new_names (dialog);
}


static void on_ok (GtkButton *button, GnomeCmdAdvrenameDialog *dialog)
{
	update_new_names (dialog);
	change_names (dialog);
	
	free_data (dialog);
	gtk_widget_hide (GTK_WIDGET (dialog));
}


static void on_cancel (GtkButton *button, GnomeCmdAdvrenameDialog *dialog)
{
	free_data (dialog);
	gtk_widget_hide (GTK_WIDGET (dialog));
}




/*******************************
 * Gtk class implementation
 *******************************/

static void
destroy (GtkObject *object)
{
	GnomeCmdAdvrenameDialog *dialog = GNOME_CMD_ADVRENAME_DIALOG (object);

	g_free (dialog->priv);
	
	if (GTK_OBJECT_CLASS (parent_class)->destroy)
		(*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}


static void
map (GtkWidget *widget)
{
	if (GTK_WIDGET_CLASS (parent_class)->map != NULL)
		GTK_WIDGET_CLASS (parent_class)->map (widget);
}


static void
class_init (GnomeCmdAdvrenameDialogClass *class)
{
	GtkObjectClass *object_class;
	GtkWidgetClass *widget_class;

	object_class = GTK_OBJECT_CLASS (class);
	widget_class = GTK_WIDGET_CLASS (class);

	parent_class = gtk_type_class (gnome_dialog_get_type ());
	object_class->destroy = destroy;
	widget_class->map = map;
}


static void
init (GnomeCmdAdvrenameDialog *in_dialog)
{
	GtkWidget *dialog;
	GtkWidget *dialog_vbox1;
	GtkWidget *vbox1;
	GtkWidget *hbox2;
	GtkWidget *scrolledwindow3;
	GtkWidget *pat_list;
	GtkWidget *label3;
	GtkWidget *label4;
	GtkWidget *vbuttonbox1;
	guint add_button_key;
	GtkWidget *add_button;
	guint remove_button_key;
	GtkWidget *remove_button;
	guint test_button_key;
	GtkWidget *test_button;
	GtkWidget *scrolledwindow4;
	GtkWidget *res_list;
	GtkWidget *label5;
	GtkWidget *label6;
	GtkWidget *ok_button;
	GtkWidget *cancel_button;
	GtkAccelGroup *accel_group;

	in_dialog->priv = g_new (GnomeCmdAdvrenameDialogPrivate, 1);

	in_dialog->priv->patterns = NULL;
	in_dialog->priv->entries = NULL;

	accel_group = gtk_accel_group_new ();

	dialog = GTK_WIDGET (in_dialog);
	gtk_object_set_data (GTK_OBJECT (dialog), "dialog", dialog);
	gtk_window_set_default_size (GTK_WINDOW (dialog), 500, 400);
	gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
	gtk_window_set_title (GTK_WINDOW (dialog), _("Advanced rename tool"));

	dialog_vbox1 = GNOME_DIALOG (dialog)->vbox;
	gtk_object_set_data (GTK_OBJECT (dialog), "dialog_vbox1", dialog_vbox1);
	gtk_widget_show (dialog_vbox1);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox1);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "vbox1", vbox1,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (hbox2);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "hbox2", hbox2,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox2, FALSE, TRUE, 4);

	scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (scrolledwindow3);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "scrolledwindow3", scrolledwindow3,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (scrolledwindow3);
	gtk_box_pack_start (GTK_BOX (hbox2), scrolledwindow3, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow3), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	pat_list = gtk_clist_new (2);
	gtk_widget_ref (pat_list);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "pat_list", pat_list,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (pat_list);
	gtk_container_add (GTK_CONTAINER (scrolledwindow3), pat_list);
	gtk_clist_set_column_width (GTK_CLIST (pat_list), 0, 194);
	gtk_clist_set_column_width (GTK_CLIST (pat_list), 1, 80);
	gtk_clist_column_titles_show (GTK_CLIST (pat_list));

	label3 = gtk_label_new (_("Replace this"));
	gtk_widget_ref (label3);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "label3", label3,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label3);
	gtk_clist_set_column_widget (GTK_CLIST (pat_list), 0, label3);

	label4 = gtk_label_new (_("With this"));
	gtk_widget_ref (label4);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "label4", label4,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label4);
	gtk_clist_set_column_widget (GTK_CLIST (pat_list), 1, label4);

	vbuttonbox1 = gtk_vbutton_box_new ();
	gtk_widget_ref (vbuttonbox1);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "vbuttonbox1", vbuttonbox1,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbuttonbox1);
	gtk_box_pack_start (GTK_BOX (hbox2), vbuttonbox1, FALSE, TRUE, 0);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (vbuttonbox1), GTK_BUTTONBOX_START);

	add_button = gtk_button_new_with_label ("");
	add_button_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (add_button)->child),
											_("_Add"));
	gtk_widget_add_accelerator (add_button, "clicked", accel_group,
								add_button_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
	gtk_widget_ref (add_button);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "add_button", add_button,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (add_button);
	gtk_container_add (GTK_CONTAINER (vbuttonbox1), add_button);
	GTK_WIDGET_SET_FLAGS (add_button, GTK_CAN_DEFAULT);

	remove_button = gtk_button_new_with_label ("");
	remove_button_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (remove_button)->child),
											   _("_Remove"));
	gtk_widget_add_accelerator (remove_button, "clicked", accel_group,
								remove_button_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
	gtk_widget_ref (remove_button);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "remove_button", remove_button,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (remove_button);
	gtk_container_add (GTK_CONTAINER (vbuttonbox1), remove_button);
	GTK_WIDGET_SET_FLAGS (remove_button, GTK_CAN_DEFAULT);

	test_button = gtk_button_new_with_label ("");
	test_button_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (test_button)->child),
											 _("_Test"));
	gtk_widget_add_accelerator (test_button, "clicked", accel_group,
								test_button_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
	gtk_widget_ref (test_button);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "test_button", test_button,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (test_button);
	gtk_container_add (GTK_CONTAINER (vbuttonbox1), test_button);
	GTK_WIDGET_SET_FLAGS (test_button, GTK_CAN_DEFAULT);

	scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (scrolledwindow4);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "scrolledwindow4", scrolledwindow4,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (scrolledwindow4);
	gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow4, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow4), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	res_list = gtk_clist_new (2);
	gtk_widget_ref (res_list);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "res_list", res_list,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (res_list);
	gtk_container_add (GTK_CONTAINER (scrolledwindow4), res_list);
	gtk_clist_set_column_width (GTK_CLIST (res_list), 0, 202);
	gtk_clist_set_column_width (GTK_CLIST (res_list), 1, 80);
	gtk_clist_column_titles_show (GTK_CLIST (res_list));

	label5 = gtk_label_new (_("Current filenames"));
	gtk_widget_ref (label5);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "label5", label5,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label5);
	gtk_clist_set_column_widget (GTK_CLIST (res_list), 0, label5);

	label6 = gtk_label_new (_("New filenames"));
	gtk_widget_ref (label6);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "label6", label6,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label6);
	gtk_clist_set_column_widget (GTK_CLIST (res_list), 1, label6);

	gnome_dialog_append_button (GNOME_DIALOG (dialog), GNOME_STOCK_BUTTON_OK);
	ok_button = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog)->buttons)->data);
	gtk_widget_ref (ok_button);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "ok_button", ok_button,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (ok_button);
	GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);

	gnome_dialog_append_button (GNOME_DIALOG (dialog), GNOME_STOCK_BUTTON_CANCEL);
	cancel_button = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog)->buttons)->data);
	gtk_widget_ref (cancel_button);
	gtk_object_set_data_full (GTK_OBJECT (dialog), "cancel_button", cancel_button,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (cancel_button);
	GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);

	gtk_signal_connect (GTK_OBJECT (add_button), "clicked",
						GTK_SIGNAL_FUNC (on_add),
						dialog);
	gtk_signal_connect (GTK_OBJECT (remove_button), "clicked",
						GTK_SIGNAL_FUNC (on_remove),
						dialog);
	gtk_signal_connect (GTK_OBJECT (test_button), "clicked",
						GTK_SIGNAL_FUNC (on_test),
						dialog);
	gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
						GTK_SIGNAL_FUNC (on_ok),
						dialog);
	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
						GTK_SIGNAL_FUNC (on_cancel),
						dialog);

	gtk_window_add_accel_group (GTK_WINDOW (dialog), accel_group);
}




/***********************************
 * Public functions
 ***********************************/

GtkWidget*
gnome_cmd_advrename_dialog_new (GList *files)
{
	GList *tmp;
	GnomeCmdAdvrenameDialog *dialog = gtk_type_new (gnome_cmd_advrename_dialog_get_type ());

	dialog->priv->files = gnome_cmd_file_list_copy (files);
	tmp = dialog->priv->files;

	while (tmp) {
		GnomeCmdFile *finfo = (GnomeCmdFile*)tmp->data;
		if (strcmp (finfo->info->name, "..") != 0)
			add_rename_entry (dialog, finfo);
		tmp = tmp->next;
	}
	
	return GTK_WIDGET (dialog);
}



GtkType
gnome_cmd_advrename_dialog_get_type         (void)
{
	static GtkType dlg_type = 0;

	if (dlg_type == 0)
	{
		GtkTypeInfo dlg_info =
		{
			"GnomeCmdAdvrenameDialog",
			sizeof (GnomeCmdAdvrenameDialog),
			sizeof (GnomeCmdAdvrenameDialogClass),
			(GtkClassInitFunc) class_init,
			(GtkObjectInitFunc) init,
			/* reserved_1 */ NULL,
			/* reserved_2 */ NULL,
			(GtkClassInitFunc) NULL
		};

		dlg_type = gtk_type_unique (gnome_dialog_get_type (), &dlg_info);
	}
	return dlg_type;
}



