/*
    GNOME Commander - A GNOME based file manager 
    Copyright (C) 2001-2002 Marcus Bjurman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/ 
#include <config.h>
#include "gnome-cmd-includes.h"


struct _GnomeCmdPatternselDialogPrivate
{
	gboolean mode;
};


static GnomeCmdStringDialogClass *parent_class = NULL;



static void
on_ok (const gchar *string, GnomeCmdPatternselDialog *dialog)
{
	if (dialog->priv->mode) {
		gnome_cmd_file_selector_select_pattern (
			gnome_cmd_main_win_get_active_fs (main_win),
			string);
	}
	else {
		gnome_cmd_file_selector_unselect_pattern (
			gnome_cmd_main_win_get_active_fs (main_win),
			string);
	}
}


static void
on_cancel (GtkWidget *widget, GnomeCmdPatternselDialog *dialog)
{
}


/*******************************
 * Gtk class implementation
 *******************************/

static void
destroy (GtkObject *object)
{
	GnomeCmdPatternselDialog *dialog = GNOME_CMD_PATTERNSEL_DIALOG (object);

	g_free (dialog->priv);
	
	if (GTK_OBJECT_CLASS (parent_class)->destroy)
		(*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}


static void
map (GtkWidget *widget)
{
	if (GTK_WIDGET_CLASS (parent_class)->map != NULL)
		GTK_WIDGET_CLASS (parent_class)->map (widget);
}


static void
class_init (GnomeCmdPatternselDialogClass *class)
{
	GtkObjectClass *object_class;
	GtkWidgetClass *widget_class;

	object_class = GTK_OBJECT_CLASS (class);
	widget_class = GTK_WIDGET_CLASS (class);

	parent_class = gtk_type_class (gnome_cmd_string_dialog_get_type ());
	object_class->destroy = destroy;
	widget_class->map = map;
}


static void
init (GnomeCmdPatternselDialog *dialog)
{
	dialog->priv = g_new (GnomeCmdPatternselDialogPrivate, 1);
}




/***********************************
 * Public functions
 ***********************************/

GtkWidget*
gnome_cmd_patternsel_dialog_new (gboolean mode)
{	
	GnomeCmdPatternselDialog *dialog = gtk_type_new (gnome_cmd_patternsel_dialog_get_type ());

	dialog->priv->mode = mode;
	
	gnome_cmd_string_dialog_set_title (
		GNOME_CMD_STRING_DIALOG (dialog),
		mode?_("Select using pattern"):_("Unselect using pattern"));
	gnome_cmd_string_dialog_set_prompt (GNOME_CMD_STRING_DIALOG (dialog), _("Enter pattern"));
//	gnome_cmd_string_dialog_set_pixmap (GNOME_CMD_STRING_DIALOG (dialog), PIXMAP_MKDIR);
	gnome_cmd_string_dialog_set_ok_cb (GNOME_CMD_STRING_DIALOG (dialog), (GnomeStringCallback)on_ok);
	gnome_cmd_string_dialog_set_cancel_cb (GNOME_CMD_STRING_DIALOG (dialog), (GnomeStringCallback)on_cancel);
	gnome_cmd_string_dialog_set_userdata (GNOME_CMD_STRING_DIALOG (dialog), dialog);
	
	return GTK_WIDGET (dialog);
}



GtkType
gnome_cmd_patternsel_dialog_get_type         (void)
{
	static GtkType dlg_type = 0;

	if (dlg_type == 0)
	{
		GtkTypeInfo dlg_info =
		{
			"GnomeCmdPatternselDialog",
			sizeof (GnomeCmdPatternselDialog),
			sizeof (GnomeCmdPatternselDialogClass),
			(GtkClassInitFunc) class_init,
			(GtkObjectInitFunc) init,
			/* reserved_1 */ NULL,
			/* reserved_2 */ NULL,
			(GtkClassInitFunc) NULL
		};

		dlg_type = gtk_type_unique (gnome_cmd_string_dialog_get_type (), &dlg_info);
	}
	return dlg_type;
}



