#include <config.h>
#include "gnome-cmd-includes.h"


#ifdef HAVE_LOCALE_H
extern struct lconv *locale_information;
#endif

#define STRINGS_TO_URIS_CHUNK 1024

void debug_print (const gchar *fmt, ...)
{
#ifdef __DEBUG__
    va_list     argptr;
    char        string[1024];

    va_start (argptr,fmt);
    vsnprintf (string,1024,fmt,argptr);
    va_end (argptr);
    g_printerr("DEBUG: %s", string);
#endif //__DEBUG__
}


void warn_print (const gchar *fmt, ...)
{
    va_list     argptr;
    char        string[1024];

    va_start (argptr,fmt);
    vsnprintf (string,1024,fmt,argptr);
    va_end (argptr);
    g_printerr("WARNING: %s", string);
}


void run_command (const gchar *command)
{
	gnome_execute_shell (NULL, command);
}


void run_shell_command_indir (const gchar *dir, const gchar *command)
{
	gnome_execute_shell (dir, command);
}


void create_error_dialog (const gchar *msg, ...)
{
    va_list     argptr;
    gchar        string[1024];

    va_start (argptr, msg);
    vsprintf (string, msg, argptr);
    va_end (argptr);

    gnome_error_dialog (string);
}


void create_warning_dialog (const gchar *msg, ...)
{
    va_list     argptr;
    gchar        string[1024];

    va_start (argptr, msg);
    vsprintf (string, msg, argptr);
    va_end (argptr);

    gnome_warning_dialog (string);
}


void create_question_dialog (const gchar *msg, gpointer callback)
{
    gnome_question_dialog_modal (msg, (GnomeReplyCallback)callback, NULL);
}



gboolean string2int (const gchar *s, gint *i)
{
    int ret;
    ret = sscanf (s, "%d", i);
    return (ret == 1);
}


gboolean string2uint (const gchar *s, guint *i)
{
    int ret;
    ret = sscanf (s, "%d", i);
    return (ret == 1);
}


gboolean string2short (const gchar *s, gshort *sh)
{
    int i,ret;
    ret = sscanf (s, "%d", &i);
    *sh = i;    
    return (ret == 1);
}


gboolean string2ushort (const gchar *s, gushort *sh)
{
    int i,ret;
    ret = sscanf (s, "%d", &i);
    *sh = i;    
    return (ret == 1);
}


gboolean string2char (const gchar *s, gchar *c)
{
    int i, ret;
    ret = sscanf (s, "%d", &i);
    *c = i;
    return (ret == 1);
}


gboolean string2uchar (const gchar *s, guchar *c)
{
    int i, ret;
    ret = sscanf (s, "%d", &i);
    *c = i;
    return (ret == 1);
}


gboolean string2float (const gchar *s, gfloat *f)
{
    int ret;
    ret = sscanf (s, "%f", f);
    return (ret == 1);
}


char *int2string (int i)
{
    return g_strdup_printf ("%d", i);
}



gchar *concat_dir_file (const gchar *dir, const gchar *file)
{
	if (!(dir && file))
		return NULL;

	if (strcmp (dir, "/") == 0)
		return g_strdup_printf ("/%s", file);

	if (dir[strlen (dir) - 1] == '/')
		return g_strdup_printf ("%s%s", dir, file);

	return g_strdup_printf ("%s/%s", dir, file);
}


gchar *str_uri_basename (const gchar *uri)
{
	int i, len, last_slash=0;

	if (!uri)
		return NULL;

	len = strlen (uri);

	if (len < 2)
		return NULL;

	for ( i=0 ; i<len ; i++ )
	{
		if (uri[i] == '/')
			last_slash = i;
	}

	return gnome_vfs_unescape_string (&uri[last_slash+1], NULL);
}          


void
type2string (GnomeVFSFileType type,
			 gchar *buf,
			 guint max)
{
	char *s;
	
	switch (type) {
	case GNOME_VFS_FILE_TYPE_UNKNOWN:
		s = "?";
		break;
	case GNOME_VFS_FILE_TYPE_REGULAR:
		s = " ";
		break;
	case GNOME_VFS_FILE_TYPE_DIRECTORY:
		s = "/";
		break;
	case GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK:
		s = "@";
		break;
	case GNOME_VFS_FILE_TYPE_FIFO:
		s = "F";
		break;
	case GNOME_VFS_FILE_TYPE_SOCKET:
		s = "S";
		break;
	case GNOME_VFS_FILE_TYPE_CHARACTER_DEVICE:
		s = "C";
		break;
	case GNOME_VFS_FILE_TYPE_BLOCK_DEVICE:
		s = "B";
		break;
	default:
		 s = "?";
	}

	g_snprintf (buf, max, "%s", s);
}


GdkPixmap *type2pixmap (GnomeVFSFileType type, gboolean big)
{
	if (big)
	{
		switch (type)
		{
			case GNOME_VFS_FILE_TYPE_UNKNOWN:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_UNKNOWN_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_REGULAR:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_REGULAR_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_DIRECTORY:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_DIRECTORY_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_SYMLINK_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_FIFO:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_FIFO_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_SOCKET:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_SOCKET_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_CHARACTER_DEVICE:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_CHAR_DEVICE_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_BLOCK_DEVICE:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_BLOCK_DEVICE_BIG);
				break;
		}
	}
	else
	{
		switch (type)
		{
			case GNOME_VFS_FILE_TYPE_UNKNOWN:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_UNKNOWN);
				break;
			case GNOME_VFS_FILE_TYPE_REGULAR:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_REGULAR);
				break;
			case GNOME_VFS_FILE_TYPE_DIRECTORY:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_DIRECTORY);
				break;
			case GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_SYMLINK);
				break;
			case GNOME_VFS_FILE_TYPE_FIFO:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_FIFO);
				break;
			case GNOME_VFS_FILE_TYPE_SOCKET:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_SOCKET);
				break;
			case GNOME_VFS_FILE_TYPE_CHARACTER_DEVICE:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_CHAR_DEVICE);
				break;
			case GNOME_VFS_FILE_TYPE_BLOCK_DEVICE:
				return IMAGE_get_pixmap (PIXMAP_FILE_TYPE_BLOCK_DEVICE);
				break;
		}
	}

	return NULL;
}


GdkBitmap *type2mask (GnomeVFSFileType type, gboolean big)
{
	if (big)
	{
		switch (type)
		{
			case GNOME_VFS_FILE_TYPE_UNKNOWN:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_UNKNOWN_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_REGULAR:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_REGULAR_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_DIRECTORY:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_DIRECTORY_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_SYMLINK_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_FIFO:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_FIFO_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_SOCKET:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_SOCKET_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_CHARACTER_DEVICE:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_CHAR_DEVICE_BIG);
				break;
			case GNOME_VFS_FILE_TYPE_BLOCK_DEVICE:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_BLOCK_DEVICE_BIG);
				break;
		}
	}
	else
	{
		switch (type)
		{
			case GNOME_VFS_FILE_TYPE_UNKNOWN:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_UNKNOWN);
				break;
			case GNOME_VFS_FILE_TYPE_REGULAR:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_REGULAR);
				break;
			case GNOME_VFS_FILE_TYPE_DIRECTORY:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_DIRECTORY);
				break;
			case GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_SYMLINK);
				break;
			case GNOME_VFS_FILE_TYPE_FIFO:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_FIFO);
				break;
			case GNOME_VFS_FILE_TYPE_SOCKET:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_SOCKET);
				break;
			case GNOME_VFS_FILE_TYPE_CHARACTER_DEVICE:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_CHAR_DEVICE);
				break;
			case GNOME_VFS_FILE_TYPE_BLOCK_DEVICE:
				return IMAGE_get_mask (PIXMAP_FILE_TYPE_BLOCK_DEVICE);
				break;
		}
	}

	return NULL;
}


GdkPixmap *mime2pixmap (const gchar *mime_type, GnomeVFSFileType type, gboolean big)
{
	GdkPixmap *pm;
	GdkBitmap *bm;
	
	if (IMAGE_get_mime_pixmap_and_mask (type, mime_type, &pm, &bm, big))
		return pm;
	else
		return type2pixmap (type, big);
}


GdkBitmap *mime2mask (const gchar *mime_type, GnomeVFSFileType type, gboolean big)
{
	GdkPixmap *pm;
	GdkBitmap *bm;
	
	if (IMAGE_get_mime_pixmap_and_mask (type, mime_type, &pm, &bm, big))
		return bm;
	else
		return type2mask (type, big);
}


void name2string (gchar *filename, gchar *buf, guint max)
{
	g_snprintf (buf, max, "%s", filename);
}


void perm2string (GnomeVFSFilePermissions p, gchar *buf, guint max)
{
	g_snprintf (buf, max, "%s%s%s%s%s%s%s%s%s", 
				(p & GNOME_VFS_PERM_USER_READ) ? "r" : "-",
				(p & GNOME_VFS_PERM_USER_WRITE) ? "w" : "-",
				(p & GNOME_VFS_PERM_USER_EXEC) ? "x" : "-",
				(p & GNOME_VFS_PERM_GROUP_READ) ? "r" : "-",
				(p & GNOME_VFS_PERM_GROUP_WRITE) ? "w" : "-",
				(p & GNOME_VFS_PERM_GROUP_EXEC) ? "x" : "-",
				(p & GNOME_VFS_PERM_OTHER_READ) ? "r" : "-",
				(p & GNOME_VFS_PERM_OTHER_WRITE) ? "w" : "-",
				(p & GNOME_VFS_PERM_OTHER_EXEC) ? "x" : "-");

}


void size2string (GnomeVFSFileSize size, gchar *buf, gint max,
				  GnomeCmdSizeDispMode size_disp_mode)
{
    if (size_disp_mode == GNOME_CMD_SIZE_DISP_MODE_POWERED)
    {
        gchar *prefixes[5] = {"b","K","M","G","T"};
        gint i=0;
        gdouble dsize = (gdouble)size;
        
        for ( i=0 ; i<5 ; i++ )
        {
            if (dsize > 1024)
                dsize /= 1024;
            else
                break;
        }

        if (i)
            g_snprintf (buf, max, "%.1f %s",dsize,prefixes[i]);
		else
			g_snprintf (buf, max, "%lld %s", size, prefixes[0]);
    }
    else if (size_disp_mode == GNOME_CMD_SIZE_DISP_MODE_GROUPED)
    {
        int i,j,len,outlen;
        char tmp[256];
        char *out;

        sprintf (tmp, "%lld", size);
        len = strlen (tmp);
        
        if (len < 4)
		{
            strncpy (buf, tmp, max);
			return;
		}
            
        outlen = len/3 + len;
		
        if ((len/3)*3 == len)
			outlen--;
           
        out = buf;
        memset (out, '\0', max);
         
        for ( i=len-1,j=outlen-1 ; i>=0 ; i--,j-- )
        {
            if (((outlen-j)/4)*4 == outlen-j)
            {
#ifdef HAVE_LOCALE_H
				out[j] = locale_information->thousands_sep[0];
#else
				out[j] = ',';
#endif
                i++;
            }
            else
                out[j] = tmp[i];
        }
    }
	else
	{
		g_snprintf (buf, max, "%lld", size);
	}
}


void time2string (time_t t, gchar *buf, guint max, const gchar *date_format)
{
    struct tm *lt;

	lt = localtime (&t);
    strftime (buf, max, date_format, lt);
}


static void
no_mime_app_found_error (gchar *mime_type)
{
	gchar *msg;

	msg = g_strdup_printf (_("No default application found for the mime-type %s.\nOpen the \"File types and programs\" page in the Control Center to add one."),
						   mime_type);
	gnome_error_dialog (msg);
}


void mime_exec_single (GnomeCmdFile *finfo)
{
	gchar *cmd, *tmp;
	GnomeVFSMimeApplication *app;
	
	g_return_if_fail (finfo != NULL);
	g_return_if_fail (finfo->info != NULL);

	if (!finfo->info->mime_type)
		return;

	app = gnome_vfs_mime_get_default_application (finfo->info->mime_type);

	if (!app) {
		no_mime_app_found_error (finfo->info->mime_type);
		return;
	}

	if (!app->expects_uris && !gnome_vfs_uri_is_local (gnome_cmd_file_get_uri (finfo))) {
		gchar *msg = g_strdup_printf ("Can't open this file since the default application \nfor this mime-type (%s) can only handle local files.", app->name);
		gnome_error_dialog (msg);
		g_free (msg);
		gnome_vfs_mime_application_free (app);
		return;
	}
	
	if (app->expects_uris && !gnome_vfs_uri_is_local (gnome_cmd_file_get_uri (finfo)))
		cmd = g_strdup_printf ("%s \"%s\"", app->command, gnome_cmd_file_get_uri_str (finfo));
	else
		cmd = g_strdup_printf ("%s \"%s\"", app->command, gnome_cmd_file_get_path (finfo));

	if (app->requires_terminal) {
		tmp = cmd;
		cmd = g_strdup_printf (gnome_cmd_data_get_term (), tmp);
		g_free (tmp);
	}
	
	run_command (cmd);
	g_free (cmd);
	gnome_vfs_mime_application_free (app);
}


void mime_exec_multiple (GList *files, GnomeVFSMimeApplication *app)
{
	gchar *cmd, *tmp;
	
	g_return_if_fail (files != NULL);
	g_return_if_fail (app != NULL);

	cmd = g_strdup_printf ("%s ", app->command);

	while (files) {
		GnomeCmdFile *finfo = (GnomeCmdFile*)files->data;
		tmp = cmd;
		if (app->expects_uris && !gnome_vfs_uri_is_local (gnome_cmd_file_get_uri (finfo)))
			cmd = g_strdup_printf ("%s \"%s\" ", tmp, gnome_cmd_file_get_uri_str (finfo));
		else
			cmd = g_strdup_printf ("%s \"%s\" ", tmp, gnome_cmd_file_get_path (finfo));
		g_free (tmp);
		files = files->next;
	}

	if (app->requires_terminal) {
		tmp = cmd;
		cmd = g_strdup_printf (gnome_cmd_data_get_term (), tmp);
		g_free (tmp);
	}

	run_command (cmd);
	g_free (cmd);
}


gboolean state_is_blank (gint state)
{
	gboolean ret;

	ret = (state & GDK_SHIFT_MASK) || (state & GDK_CONTROL_MASK) || (state & GDK_MOD1_MASK);
	
	return !ret;
}


gboolean state_is_shift (gint state)
{
	return (state & GDK_SHIFT_MASK) && !(state & GDK_CONTROL_MASK) && !(state & GDK_MOD1_MASK);
}


gboolean state_is_ctrl (gint state)
{
	return !(state & GDK_SHIFT_MASK) && (state & GDK_CONTROL_MASK) && !(state & GDK_MOD1_MASK);
}


gboolean state_is_alt (gint state)
{
	return !(state & GDK_SHIFT_MASK) && !(state & GDK_CONTROL_MASK) && (state & GDK_MOD1_MASK);
}


gboolean state_is_alt_shift (gint state)
{
	return (state & GDK_SHIFT_MASK) && !(state & GDK_CONTROL_MASK) && (state & GDK_MOD1_MASK);
}


gboolean state_is_ctrl_alt (gint state)
{
	return !(state & GDK_SHIFT_MASK) && (state & GDK_CONTROL_MASK) && (state & GDK_MOD1_MASK);
}


gboolean state_is_ctrl_shift (gint state)
{
	return (state & GDK_SHIFT_MASK) && (state & GDK_CONTROL_MASK) && !(state & GDK_MOD1_MASK);
}


void clear_event_key (GdkEventKey *event)
{
	g_return_if_fail (event != NULL);
	g_return_if_fail (event->string != NULL);
	
	event->keyval = GDK_VoidSymbol;
	event->string[0] = '\0';	
}


/**
 * Transform a "\r\n" separated string into a GList with GnomeVFSURI's
 */
GList *
strings_to_uris (gchar *data)
{
	int i;
	GList *uri_list = NULL;
	gchar **filenames;

	filenames = g_strsplit (data, "\r\n", STRINGS_TO_URIS_CHUNK);
	
	for ( i=0 ; filenames[i] != NULL ; i++ ) {
		gchar *fn;
		
		if (i == STRINGS_TO_URIS_CHUNK) {
			uri_list = g_list_concat (uri_list, strings_to_uris (filenames[i]));
			break;
		}

		fn = g_strdup (filenames[i]);
		uri_list = g_list_append (uri_list, gnome_vfs_uri_new (fn));
		g_free (fn);
	}

	g_strfreev (filenames);
	return uri_list;
}


GnomeVFSFileSize calc_tree_size (const GnomeVFSURI *dir_uri)
{
	GnomeVFSFileSize size = 0;
	GnomeVFSFileInfoOptions infoOpts = 0;
	GList *list, *tmp;
	gchar *dir_uri_str;
	GnomeVFSResult result;

	if (!dir_uri) return 0;

	dir_uri_str = gnome_vfs_uri_to_string (dir_uri, 0);
		
	if (!dir_uri_str) return 0;
	
	result = gnome_vfs_directory_list_load (
		&list,
		dir_uri_str,
		infoOpts,
		NULL);

	tmp = list;
	while (tmp) {
		GnomeVFSFileInfo *info = (GnomeVFSFileInfo*)tmp->data;
		if (strcmp (info->name, ".") != 0 && strcmp (info->name, "..") != 0) {		
			if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
				GnomeVFSURI *new_dir_uri = gnome_vfs_uri_append_file_name (
					dir_uri, info->name);
				size += calc_tree_size (new_dir_uri);
				gnome_vfs_uri_unref (new_dir_uri);
			}
			else
				size += info->size;
		}

		tmp = tmp->next;
	}

	tmp = list;
	while (tmp) {
		GnomeVFSFileInfo *info = (GnomeVFSFileInfo*)tmp->data;		
		gnome_vfs_file_info_unref (info);		
		tmp = tmp->next;
	}
	
	g_list_free (list);
	g_free (dir_uri_str);

	return size;
}


